from .dev_config import DevelopmentConfig
from .prod_config import ProductionConfig
from .local_config import LocalConfig

profiles = {
    'dev': DevelopmentConfig,
    'prod': ProductionConfig,
    'default': LocalConfig
}


def active_config_by_profile(active_profile, command_args):
    global config
    config = profiles[active_profile]
    config.init_config(command_args)



import json
from unittest import mock

from app.assets import assets_repository
from app.assets import assets_service
from app.assets.asset_model import AssetModel
from app.assets.asset_summary_model import AssetsSummaryModel
from app.assets.builder.AssetModelBuilder import AssetModelBuilder
from app.foundation.constants import constant


@mock.patch.object(assets_repository, 'get_assets_summary')
def test_should_return_asset_summary(get_assets_summary):
    assets_summary_model = AssetsSummaryModel()
    assets_summary_model.client_no = 'client-no-foo'
    assets_summary_model.currencies = {"as_btc": 1.5, "as_usd": 1.6}
    assets_summary_model.reference_no = '1521180097'
    assets_summary_model.clearing_time = '2018-03-16 06:01:44'

    get_assets_summary.return_value = assets_summary_model

    is_success, asset_summary_result, description = assets_service.get_assets_summary(None)

    assert is_success is True
    assert asset_summary_result.currencies["as_btc"] == 1.5
    assert asset_summary_result.currencies["as_usd"] == 1.6
    assert asset_summary_result.reference_no is '1521180097'
    assert asset_summary_result.clearing_time is '2018-03-16 06:01:44'


@mock.patch.object(assets_repository, 'get_assets_summary')
def test_should_return_empty_total_assets_amount_if_no_assets_was_found(get_assets_summary):
    get_assets_summary.return_value = None

    is_success, assets_summary_result, description = assets_service.get_assets_summary(None)

    assert is_success is True

    currencies = json.loads(assets_summary_result.currencies)
    assert currencies["as_btc"] == 0.0
    assert currencies["as_usd"] == 0.0
    assert description is constant.NO_ASSET_SUMMARY_WAS_FOUND
    assert assets_summary_result.reference_no is None
    assert assets_summary_result.clearing_time is None


@mock.patch.object(assets_repository, 'get_asset_models')
def test_should_return_assets(get_asset_models):
    given_client_no = 'client-no-foo'
    expected_asset_models = []
    asset_model = AssetModelBuilder() \
        .with_currency('BTC') \
        .with_origin_balance(0.6) \
        .with_current_balance(0.8) \
        .with_as_btc(0.8) \
        .with_as_usd(7735.03) \
        .with_reference_no('1234567') \
        .end()

    expected_asset_models.append(asset_model)
    get_asset_models.return_value = expected_asset_models

    is_success, assets_result, description = assets_service.get_assets(given_client_no)

    assert is_success is True
    assert len(assets_result) == 1

    first_asset = assets_result[0]
    assert first_asset.currency is 'BTC'
    assert first_asset.origin_balance == 0.600000
    assert first_asset.current_balance == 0.800000
    assert first_asset.earnings == 0.200000
    assert first_asset.roa == 33.33
    assert first_asset.as_usd == 7735.03
    assert first_asset.reference_no is '1234567'


@mock.patch.object(assets_repository, 'get_asset_models')
def test_should_return_empty_assets_if_no_assets_was_found(get_asset_models):
    get_asset_models.return_value = None

    is_success, assets_result, description = assets_service.get_assets(None)

    assert is_success is True
    assert not assets_result
    assert description is constant.NO_ASSETS_WAS_FOUND


@mock.patch.object(assets_repository, 'get_currencies')
def test_should_return_empty_currencies_if_no_currencies_were_found(get_currencies):
    get_currencies.return_value = None

    is_success, currencies, description = assets_service.get_currencies(None)

    assert is_success is True
    assert not currencies
    assert description is constant.NO_CURRENCIES_WERE_FOUND


@mock.patch.object(assets_repository, 'get_currencies')
def test_should_return_currencies(get_currencies):
    expected_currency_models = []
    currency_model = AssetModel()
    currency_model.currency = 'BTC'
    expected_currency_models.append(currency_model)
    get_currencies.return_value = expected_currency_models

    is_success, currencies, description = assets_service.get_currencies('client_no-foo')

    assert is_success is True
    assert currencies[0].name is 'BTC'



from .config import BasicConfig


class ProductionConfig(BasicConfig):
    SQLALCHEMY_DATABASE_URI = 'mysql://root:000000@localhost/assets_center'
    SQLALCHEMY_TRACK_MODIFICATIONS = False

    JWT_SECRET_KEY = 'super-secret'
    CRYPTO_KEY = 'U70eDgRUKo1D4bCJcsyBkGVDLrBWFrJRsHdqjIs_CiE='



# content of test_class.py
from unittest import mock

from app.auth import auth
from app.foundation.constants import constant
from app.users import users_repository
from app.users.user_model import UserModel


def test_should_return_if_identification_is_empty():
    is_success, token, description = auth.authenticate(None, 'password')
    assert is_success is False
    assert description is constant.IDENTIFICATION_IS_REQUIRED


def test_should_return_if_password_is_empty():
    is_success, token, description = auth.authenticate('identification', None)
    assert is_success is False
    assert description is constant.PASSWORD_IS_REQUIRED


@mock.patch.object(users_repository, 'get_user_by_identification')
@mock.patch.object(auth, 'decrypt_password')
@mock.patch.object(auth, 'create_access_token')
def test_should_return_token_if_authenticate_successfully(create_access_token, decrypt_password,
                                                          get_user_by_identification):
    user_model = UserModel()
    user_model.client_no = 'client_no-foo'

    get_user_by_identification.return_value = user_model
    decrypt_password.return_value = '123456'
    create_access_token.return_value = 'token-foo'

    is_success, token, description = auth.authenticate('17512345678', '123456')

    assert is_success is True
    assert token is 'token-foo'


@mock.patch.object(users_repository, 'get_user_by_identification')
def test_should_return_if_user_does_not_exist(get_user_by_identification):
    get_user_by_identification.return_value = None

    is_success, token, description = auth.authenticate('15712345678', '123456')

    assert is_success is False
    assert description is constant.USER_DOES_NOT_EXIST


@mock.patch.object(auth, 'decrypt_password')
@mock.patch.object(users_repository, 'get_user_by_identification')
def test_should_return_if_identification_or_password_is_incorrect(get_user_by_identification, decrypt_password):
    user_model = UserModel()
    user_model.client_no = 'client_no-foo'

    get_user_by_identification.return_value = user_model
    decrypt_password.return_value = '123456'

    is_success, token, description = auth.authenticate('15712345678', '234567')
    assert is_success is False
    assert description is constant.USER_OR_PASSWORD_DOES_NOT_CORRECT


@mock.patch.object(users_repository, 'get_user_by_identification')
@mock.patch.object(auth, 'decrypt_password')
@mock.patch.object(auth, 'safe_str_cmp')
def test_should_return_password_is_incorrect(safe_str_cmp, decrypt_password, get_user_by_identification):
    user_model = UserModel()
    user_model.client_no = 'client_no-foo'

    get_user_by_identification.return_value = user_model
    decrypt_password.return_value = '123456'
    safe_str_cmp.return_value = False

    is_success, token, description = auth.authenticate('15712345678', '234567')
    assert is_success is False
    assert description is constant.USER_OR_PASSWORD_DOES_NOT_CORRECT



# coding=utf-8
import logging
from time import strftime
from logging.handlers import RotatingFileHandler
import os

application_name = 'user_center'
application_display_name = '用户中心'


class BasicConfig:
    LOG_PATH = 'logs'
    MONGO_USER = ''
    MONGO_PASSWORD = ''
    MONGO_HOST = '127.0.0.1'
    MONGO_PORT = '27017'
    MONGO_DB = 'data_warehouse'

    JWT_SECRET_KEY = ''
    JWT_HEADER_TYPE = 'JWT'
    CRYPTO_KEY = ''


    @classmethod
    def init_app(cls, app):
        if command_args.log_path is not None:
            self.LOG_PATH = command_args.log_path
        config_logging(self.LOG_PATH)
        self.MONGO_URI = 'mongodb://{}:{}@{}:{}'.format(self.MONGO_USER, quote(self.MONGO_PASSWORD), self.MONGO_HOST, self.MONGO_PORT)



def config_logging(log_path):
    if not os.path.exists(log_path):
        os.makedirs(log_path)
    if not log_path.endswith('/'):
        log_path += '/'

    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    default_handler = RotatingFileHandler(strftime(log_path + 'log.%Y_%m_%d'), maxBytes=1024 * 1024 * 5,
                                          backupCount=100)
    default_handler.setLevel(logging.INFO)
    default_handler.setFormatter(formatter)
    logger.addHandler(default_handler)

    error_handler = RotatingFileHandler(strftime(log_path + 'error.log.%Y_%m_%d'), maxBytes=1024 * 1024 * 5,
                                        backupCount=100)
    error_handler.setLevel(logging.ERROR)
    error_handler.setFormatter(formatter)
    logger.addHandler(error_handler)

    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)
    console_handler.setLevel(logging.INFO)
    logger.addHandler(console_handler)



from datetime import datetime
from unittest import mock

from app.earnings import earnings_service, earnings_repository
from app.earnings.earnings_model import EarningsModel
from app.foundation.constants import constant


@mock.patch.object(earnings_repository, 'get_earnings_statistics')
def test_should_return_empty_if_no_currency_found(get_earnings_statistics):
    get_earnings_statistics.return_value = None

    is_success, history_earnings, description = earnings_service.get_history_earnings('client_no-foo', 'currency-foo',
                                                                                      '2018-03-01', '2018-03-30')

    assert is_success is True
    assert description is constant.NO_EARNINGS_WAS_FOUND
    assert history_earnings is None


@mock.patch.object(earnings_repository, 'get_earnings_statistics')
def test_should_return_history_earnings(get_earnings_statistics):
    expected_earnings_models = []
    earnings_model = EarningsModel()
    earnings_model.earning = 0.300000
    earnings_model.create_time = datetime.strptime('2018-03-30', '%Y-%m-%d')
    expected_earnings_models.append(earnings_model)
    get_earnings_statistics.return_value = expected_earnings_models

    is_success, history_earnings, description = earnings_service.get_history_earnings('client_no-foo', 'BTC',
                                                                                      '2018-03-01', '2018-03-30')

    assert is_success is True
    assert history_earnings.currency == 'BTC'
    assert history_earnings.earnings[0].date.strftime('%Y-%m-%d') == '2018-03-30'
    assert history_earnings.earnings[0].earning is 0.300000
    assert history_earnings.period.start_date == '2018-03-01'
    assert history_earnings.period.end_date == '2018-03-30'



from .config import BasicConfig


class LocalConfig(BasicConfig):
    LOG_PATH = 'local_logs'
    MONGO_USER = 'tao'
    MONGO_PASSWORD = '000000'

    JWT_SECRET_KEY = 'super-secret'
    CRYPTO_KEY = 'U70eDgRUKo1D4bCJcsyBkGVDLrBWFrJRsHdqjIs_CiE='



from unittest import mock

from app.users import users_repository
from app.users import users_service
from app.users.user_model import UserModel


@mock.patch.object(users_repository, 'get_profile')
def test_should_return_user_profile(get_profile):
    given_client_no = 'client-no-foo'

    user_model = UserModel()
    user_model.client_no = 'client-no-foo'
    user_model.name = 'foo-name'
    user_model.email = 'name@example.com'
    user_model.mobile = '15800000000'

    get_profile.return_value = user_model

    is_success, user_profile_result, description = users_service.get_profile(given_client_no)

    assert is_success is True
    assert user_profile_result.client_no is 'client-no-foo'
    assert user_profile_result.name is 'foo-name'
    assert user_profile_result.mobile is '15800000000'
    assert user_profile_result.email is 'name@example.com'


@mock.patch.object(users_repository, 'get_profile')
def test_should_return_none_if_no_user_profile_was_found(get_profile):
    given_client_no = 'client-no-foo'
    get_profile.return_value = None

    is_success, user_profile_result, description = users_service.get_profile(given_client_no)

    assert is_success is False
    assert user_profile_result is None
